#ifndef QTS_HTTP_FILE_STATIONS_H
#define QTS_HTTP_FILE_STATIONS_H

#include "QtsHttpComm.h"

#ifdef QTSHTTP_EXPORTS
#define QTSHTTPDLL_API __declspec(dllexport)
#define EXPIMP_TEMPLATE
#else
#define QTSHTTPDLL_API __declspec(dllimport)
#define EXPIMP_TEMPLATE extern
#endif

/** File station map to SDK version. */
typedef enum _QTS_HTTP_FILE_STATION_API_VERSION_
{
	/** Unknow version */
	QTS_HTTP_FILE_STATION_API_VERSION_UNKNOW = 0,	
	/** File station API Version 1 */
	QTS_HTTP_FILE_STATION_API_V1   = 1,					
	//QTS_HTTP_FILE_STATION_API_V2 = 2,
}QTS_HTTP_FILE_STATION_API_VERSION;

/** Compressed format for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_
{
	/** ZIP compressed format */
	QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_ZIP = 0,		
	/** 7Z compressed format */
	QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_7Z = 1,		
}QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE;

/** Compressed level for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_
{
	/** Normal compressed level */
	QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_NORMAL = 0,	
	/** The largest compressed level */
	QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_LARGE,				
	/** The fastest compressed level */
	QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_FAST,				
}QTS_HTTP_FILE_STATION_COMPRESS_LEVEL;

/** Compressed encrypt type for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_
{
	/** AES256 encrypt */
	QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_AES256 = 0,	
	/** ZIP encrypt */
	QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_ZIPCRYPTO,		
}QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE;

/** Compressed update mode for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_
{
	/** Add and replace if file exist */
	QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_ADD_REPLACE = 0,		
	/** Update and add if file exist */
	QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_ADD,
	/** Update exist file */
	QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_EXISTING,	
	/** Synchronize update file */
	QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_SYNCHRONIZE,	
}QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE;

/** Extract mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_EXTRACT_MODE_
{
	/** Extract all files */
	QTS_HTTP_FILE_STATION_EXTRACT_MODE_ALL = 0,	
	/** Extract partial files */
	QTS_HTTP_FILE_STATION_EXTRACT_MODE_PART,			
}QTS_HTTP_FILE_STATION_EXTRACT_MODE;

/** Extract path mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_
{
	/** No full path mode */
	QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_NONE = 0,	
	/** Full path mode */
	QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_FULL,			
}QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE;

/** File overwrite mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_OVERWRITE_
{
	/** Skip */
	QTS_HTTP_FILE_STATION_OVERWRITE_SKIP = 0,	
	/** Overwrite */
	QTS_HTTP_FILE_STATION_OVERWRITE_OVERWRITE,		
}QTS_HTTP_FILE_STATION_OVERWRITE;

/** File station API type for async mode response context */
typedef enum _QTS_HTTP_FILE_STATION_API_TYPE_
{
	/** Unknow API */
	QTS_HTTP_FILE_STATION_API_TYPE_UNKNOW = 0,		
	/** GetFileListCountAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETFILELISTCOUNT,			
	/** GetFileListAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETFILELIST,						
	/** GetShareFolderListAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETSHAREFOLDERLIST,				
	/** CreateShareLinkAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_CREATESHARELINK,					
	/** GetShareLinkListAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETSHARELINKLIST,	
	/** DeleteShareLinkAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_DELETESHARELINK,		
	/** CreateFolderAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_CREATEFOLDER,
	/** CopyFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_COPYFILE,			
	/** MoveFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_MOVEFILE,						
	/** DeleteFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_DELETEFILE,						
	/** UploadFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_UPLOADFILE,						
	/** DownloadFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_DOWNLOADFILE,					
	/** GetFileStatusAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETFILESTATUS,					
	/** SearchFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_SEARCH,							
	/** RenameFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_RENAME,							
	/** UpdateFileModifyDateAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_UPDATE_MODIFY_DATE,				
	/** CompressFilesAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_COMPRESSFILES,					
	/** ListContentsOfExtractedFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_LISTCONTENTS_EXTRACTEDFILE,		
	/** ExtractFilesAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_EXTRACTEDFILE,					
	/** GetDomainIPListAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_GETDOMAINIPLIST,					
	/** AddVideoTranscodeAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_ADDVIDEOTRANSCODEFILE,			
	/** DeleteVideoTranscodeAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_DELETEVIDEOTRANSCODEFILE,		
	/** CreateChunkedUploadIDAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_CREATECHUNKEDUPLOADID,			
	/** ChunkedUploadFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_CHUNKEDUPLOAD,					
	/** DeleteChunkedUploadFileAsync API */
	QTS_HTTP_FILE_STATION_API_TYPE_DELETECHUNKEDUPLOADFILE,			
}QTS_HTTP_FILE_STATION_API_TYPE;

/*!
 * \brief
 * File station callback function for sync mode API callback progress rate.
 * 
 * \param
 * [OUT] lpData : Callback function owner.
 * \param
 * [OUT] nProgressRate : Progress rate.
 * 
 * \returns
 * n/a
 */
typedef void (__stdcall *PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK)(LPVOID lpData, int nProgressRate);
/*!
 * \brief
 * File station callback function for sync mode API callback progress bytes.
 * 
 * \param
 * [OUT] lpData : Callback function owner.
 * \param
 * [OUT] llTotalBytesTransferred : Total bytes transferred.
 * 
 * \returns
 * n/a
 */
typedef void (__stdcall *PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK)(LPVOID lpData, LONGLONG llTotalBytesTransferred);

/*!
 * \brief
 * Context for file station response callback
 * 
 */
typedef struct _QTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CTX_
{
	/** QTS HTTP file station API result */
	QTS_HTTP_API_RESULT						lResult;	
	/** QTS HTTP file station API type */
	QTS_HTTP_FILE_STATION_API_TYPE			eAPIType;		
	/** QTS HTTP file station progress rate of transferred */
	int										nProgressRate;	
	/** QTS HTTP file station total bytes of transferred */
	LONGLONG								llTotalBytesTransferred;	
	/** QTS HTTP file station request */
	LPVOID									lpFSRequest;	
	/** QTS HTTP file station callback function owner */
	LPVOID									lpData;						
} QTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CTX, *PQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CTX;

/*!
 * \brief
 * File station callback function for async mode API.
 * 
 * \param
 * [OUT] PQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CTX : Context for file station response.
 * 
 * \returns
 * n/a
 */
typedef void (__stdcall *PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK)(PQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CTX);

/*!
 * \brief
 * Share link information class
 * 
 */
class QTSHTTPDLL_API CQtsHttpShareLinkInfo
{
public:
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] strShareFilePath : Share file path include filename.
	 * \param 
	 * [IN] strPassword : Share file use password or not.
	 * \param 
	 * [IN] strDateTime : Share day and hour time.
	 * \param 
	 * [IN] bUseSSL : Share file use SSL or not.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpShareLinkInfo(LPCTSTR strShareFilePath, LPCTSTR strPassword, LPCTSTR strDateTime, BOOL bUseSSL);

	/*!
	 * \brief
	 * Get path include filename of share file.
	 * 
	 * \returns
	 * LPCTSTR : Share file path include filename.
	 */
	LPCTSTR GetShareFilePath();
	/*!
	 * \brief
	 * Get password of share file.
	 * 
	 * \returns
	 * LPCTSTR : Password of share file.
	 */
	LPCTSTR GetPassword();
	/*!
	 * \brief
	 * Get share day and hour time.
	 * 
	 * \returns
	 * LPCTSTR : Share day and hour time.
	 */
	LPCTSTR GetDateTime();
	/*!
	 * \brief
	 * Get share file with SSL or not.
	 * 
	 * \returns
	 * TRUE : Share file with SSL.
	 * \returns
	 * FALSE : Share file with non-SSL.
	 */
	BOOL IsSecureMode();

	/*!
	 * \brief
	 * Set path include filename of share file.
	 * 
	 * \param
	 * [IN] strShareFilePath : Share file path include filename.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetShareFilePath(LPCTSTR strShareFilePath);
	/*!
	 * \brief
	 * Set password of share file.
	 * 
	 * \param
	 * [IN] strPassword : Password of share file.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetPassword(LPCTSTR strPassword);
	/*!
	 * \brief
	 * Set share day and hour time.
	 * 
	 * \param
	 * [IN] strDataTime : Share day and hour time.
	 * 
	 * \returns
	 * n/a.
	 * \remarks
	 * If strDataTime = 0, share forever.
	 */
	void SetDateTime(LPCTSTR strDataTime);
	/*!
	 * \brief
	 * Set share file  with SSL or not.
	 * 
	 * \param
	 * [IN] bUseSSL : Share file with SSL or not.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetIsSecureMode(BOOL bUseSSL);

private:
	wstring		m_strShareFilePath;    /** Share file path include filename */
	wstring		m_strPassword;         /** Share file with password */
	wstring		m_strDateTime;         /** Share day and hour time. If dateTime = 0, share forever. */
	BOOL		m_bUseSSL;             /** Share file with SSL */
};

/*!
 * \brief
 * File or folder information class
 * 
 */
class QTSHTTPDLL_API CQtsHttpFileEntry
{
public:
	/** Default constructor */
	CQtsHttpFileEntry();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] bDir : Is file or folder.
	 * \param 
	 * [IN] llSize : File or folder size.
	 * \param 
	 * [IN] strPath : File or folder path.
	 * \param 
	 * [IN] strName : File or folder name.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpFileEntry(  
		BOOL	   bDir,
		LONGLONG   llSize,
		LPCTSTR	   strPath,
		LPCTSTR	   strName
		);

	/*!
	 * \brief
	 * Is folder or file.
	 * 
	 * \returns
	 * TRUE : Folder entry.
	 * \returns
	 * FALSE : File entry.
	 */
	BOOL IsDir();
	/*!
	 * \brief
	 * Get file/folder size.
	 * 
	 * \returns
	 * LONGLONG : File/folder size.
	 */
	LONGLONG GetFileSize();
	/*!
	 * \brief
	 * Get file/folder path.
	 * 
	 * \returns
	 * LPCTSTR : File/folder path.
	 */
	LPCTSTR	GetFilePath();
	/*!
	 * \brief
	 * Get file/folder name.
	 * 
	 * \returns
	 * LPCTSTR : File/folder name.
	 */
	LPCTSTR	GetFileName();

	/*!
	 * \brief
	 * Set CQtsHttpFileEntry is file or folder.
	 * 
	 * \param
	 * [IN] bDir : Is file or folder.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetDir(BOOL bDir);
	/*!
	 * \brief
	 * Set file or folder size.
	 * 
	 * \param
	 * [IN] llSize : File or folder size.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFileSize(LONGLONG llSize);
	/*!
	 * \brief
	 * Set file or folder path.
	 * 
	 * \param
	 * [IN] strPath : File or folder path.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFilePath(LPCTSTR strPath);
	/*!
	 * \brief
	 * Set file or folder name.
	 * 
	 * \param
	 * [IN] strName : File or folder name.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFileName(LPCTSTR strName);

private:
	BOOL	   m_bDir;			/** Is file or folder */
	LONGLONG   m_llSize;		/** File or folder size */
	wstring	   m_strPath;		/** File or folder path */
	wstring	   m_strName;		/** File or folder name */
};

/*!
 * \brief
 * File or folder status class
 * 
 */
class QTSHTTPDLL_API CQtsHttpFileStatus
{
public:
	/** Default constructor */
	CQtsHttpFileStatus();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] bDir : Is file or folder.
	 * \param 
	 * [IN] bCommpressed : Is compressed.
	 * \param 
	 * [IN] llSize : File or folder size.
	 * \param 
	 * [IN] strFolderPermission : File or folder permission.
	 * \param 
	 * [IN] strPath : File or folder path.
	 * \param 
	 * [IN] strName : File or folder name.
	 * \param 
	 * [IN] strGroup : File or folder group.
	 * \param 
	 * [IN] strOwner : File or folder owner.
	 * \param 
	 * [IN] strMT : File or folder modification time.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpFileStatus(  
		BOOL	   bDir,
		BOOL       bCommpressed,
		LONGLONG   llSize,
		LPCTSTR    strFolderPermission,
		LPCTSTR	   strPath,
		LPCTSTR	   strName,
		LPCTSTR	   strGroup,
		LPCTSTR	   strOwner,
		LPCTSTR	   strMT
		);

	/*!
	 * \brief
	 * Is file or folder.
	 * 
	 * \returns
	 * TRUE : Folder entry.
	 * \returns
	 * FALSE : File entry.
	 */
	BOOL IsDir();
	/*!
	 * \brief
	 * Is compressed.
	 * 
	 * \returns
	 * TRUE : File or folder compressed.
	 * \returns
	 * FALSE : File or folder not compressed.
	 */
	BOOL IsCommpressed();
	/*!
	 * \brief
	 * Get file or folder size.
	 * 
	 * \returns
	 * LONGLONG : File or folder size.
	 */
	LONGLONG GetFileSize();
	/*!
	 * \brief
	 * Get file or folder permission.
	 * 
	 * \returns
	 * LPCTSTR : File or folder permission.
	 */
	LPCTSTR GetFolderPermission();
	/*!
	 * \brief
	 * Get file or folder path.
	 * 
	 * \returns
	 * LPCTSTR : File or folder path.
	 */
	LPCTSTR	GetFilePath();
	/*!
	 * \brief
	 * Get file or folder name.
	 * 
	 * \returns
	 * LPCTSTR : File or folder name.
	 */
	LPCTSTR	GetFileName();
	/*!
	 * \brief
	 * Get file or folder group.
	 * 
	 * \returns
	 * LPCTSTR : File or folder group.
	 */
	LPCTSTR	GetGroup();
	/*!
	 * \brief
	 * Get file or folder owner.
	 * 
	 * \returns
	 * LPCTSTR : File or folder owner.
	 */
	LPCTSTR	GetOwner();
	/*!
	 * \brief
	 * Get file or folder modification time.
	 * 
	 * \returns
	 * LPCTSTR : File or folder modification time.
	 */
	LPCTSTR	GetMT();

	/*!
	 * \brief
	 * Set is folder or not.
	 * 
	 * \param
	 * [IN] bDir : Is folder or not.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetDir(BOOL bDir);
	/*!
	 * \brief
	 * Set is compressed.
	 * 
	 * \param
	 * [IN] bCompressed : Is compressed.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetCompressed(BOOL bCompressed);
	/*!
	 * \brief
	 * Set file or folder size.
	 * 
	 * \param
	 * [IN] llSize : File or folder size.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFileSize(LONGLONG llSize);
	/*!
	 * \brief
	 * Set file or folder permission.
	 * 
	 * \param
	 * [IN] strFolderPermission : File or folder permission.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFolderPermission(LPCTSTR strFolderPermission);
	/*!
	 * \brief
	 * Set file or folder path.
	 * 
	 * \param
	 * [IN] strPath : File or folder path.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFilePath(LPCTSTR strPath);
	/*!
	 * \brief
	 * Set file or folder name.
	 * 
	 * \param
	 * [IN] strName : File or folder name.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFileName(LPCTSTR strName);
	/*!
	 * \brief
	 * Set file or folder group.
	 * 
	 * \param
	 * [IN] strGroup : File or folder group.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetGroup(LPCTSTR strGroup);
	/*!
	 * \brief
	 * Set file or folder owner.
	 * 
	 * \param
	 * [IN] strOwner : File or folder owner.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetOwner(LPCTSTR strOwner);
	/*!
	 * \brief
	 * Set file or folder modification time.
	 * 
	 * \param
	 * [IN] strMT : File or folder modification time.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetMT(LPCTSTR strMT);

private:
	BOOL	   m_bDir;					/** Is file or folder */
	BOOL       m_bCommpressed;			/** Is compressed */
	LONGLONG   m_llSize;				/** File or folder size */
	wstring    m_strFolderPermission;	/** File or folder permission */
	wstring	   m_strPath;				/** File or folder path */
	wstring	   m_strName;				/** File or folder name */
	wstring	   m_strGroup;				/** File or folder group */
	wstring	   m_strOwner;				/** File or folder owner */
	wstring	   m_strMT;					/** File or folder modification time */
};

/*!
 * \brief
 * Share link class
 * 
 */
class QTSHTTPDLL_API CQtsHttpShareLink
{
public:
	/** Constructor */
	CQtsHttpShareLink();

	/*!
	 * \brief
	 * Get share link file name.
	 * 
	 * \returns
	 * LPCTSTR : Share link file name.
	 */
	LPCTSTR GetFileName();
	/*!
	 * \brief
	 * Get share link expire time.
	 * 
	 * \returns
	 * LPCTSTR : Share link expire time.
	 */
	LPCTSTR GetExpireTime();
	/*!
	 * \brief
	 * Get share link URL.
	 * 
	 * \returns
	 * LPCTSTR : Share link URL.
	 */
	LPCTSTR GetURL();

	/*!
	 * \brief
	 * Set share link file name.
	 * 
	 * \param
	 * [IN] strFileName : Share link file name.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetFileName(LPCTSTR strFileName);
	/*!
	 * \brief
	 * Set share link expire time.
	 * 
	 * \param
	 * [IN] strExpireTime : Share link expire time.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetExpireTime(LPCTSTR strExpireTime);
	/*!
	 * \brief
	 * Set share link URL.
	 * 
	 * \param
	 * [IN] strURL : Share link URL.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetURL(LPCTSTR strURL);

private:
	wstring	   m_strFileName;		/** Share link file name */
	wstring	   m_strExpireTime;		/** Share link expire time */
	wstring	   m_strURL;			/** Share link URL */
};

/*!
 * \brief
 * Each file/folder entry in extracted file
 * 
 */
class QTSHTTPDLL_API CQtsHttpExtractEntry
{
public:
	/** Default constructor */
    CQtsHttpExtractEntry();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] bDir : Is file or folder.
	 * \param 
	 * [IN] llSize : File or folder size.
	 * \param 
	 * [IN] llCompressSize : File or folder compress size.
	 * \param 
	 * [IN] strModifyTime : File or folder modification time.
	 * \param 
	 * [IN] strName : File or folder full path.
	 * 
	 * \returns
	 * n/a.
	 */
    CQtsHttpExtractEntry(  
			BOOL	   bDir,
            LONGLONG   llSize,
			LONGLONG   llCompressSize,
            LPCTSTR	   strModifyTime,
            LPCTSTR	   strName
         );

	/*!
	 * \brief
	 * Is file or folder in extracted file.
	 * 
	 * \returns
	 * TRUE : Folder entry.
	 * \returns
	 * FALSE : File entry.
	 */
    BOOL IsDir();
	/*!
	 * \brief
	 * Get file or folder size in extracted file.
	 * 
	 * \returns
	 * LONGLONG : File or folder size.
	 */
    LONGLONG GetFileSize();
	/*!
	 * \brief
	 * Get file or folder compress size in extracted file.
	 * 
	 * \returns
	 * LONGLONG : File or folder compress size.
	 */
	LONGLONG GetCompressFileSize();
	/*!
	 * \brief
	 * Get file or folder modification time in extracted file.
	 * 
	 * \returns
	 * LPCTSTR : File or folder modification time.
	 */
    LPCTSTR	GetModifyTime();
	/*!
	 * \brief
	 * Get file or folder full path in extracted file.
	 * 
	 * \returns
	 * LPCTSTR : File or folder full path.
	 */
    LPCTSTR	GetFileName();

	/*!
	 * \brief
	 * Set is folder or not.
	 * 
	 * \param
	 * [IN] bDir : Is file or folder.
	 * 
	 * \returns
	 * n/a.
	 */
    void SetDir(BOOL bDir);
	/*!
	 * \brief
	 * Set file or folder size.
	 * 
	 * \param
	 * [IN] llSize : File or folder size.
	 * 
	 * \returns
	 * n/a.
	 */
    void SetFileSize(LONGLONG llSize);
	/*!
	 * \brief
	 * Set file or folder compress size.
	 * 
	 * \param
	 * [IN] llCompressSize : File or folder compress size.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetCompressFileSize(LONGLONG llCompressSize);
	/*!
	 * \brief
	 * Set file or folder modification time.
	 * 
	 * \param
	 * [IN] strMT : File or folder modification time.
	 * 
	 * \returns
	 * n/a.
	 */
    void SetModifyTime(LPCTSTR strMT);
	/*!
	 * \brief
	 * Set file or folder full path.
	 * 
	 * \param
	 * [IN] strName : File or folder full path.
	 * 
	 * \returns
	 * n/a.
	 */
    void SetFileName(LPCTSTR strName);

private:
	BOOL	   m_bDir;				/** Is file or folder */
	LONGLONG   m_llSize;			/** File or folder size */
	LONGLONG   m_llCompressSize;	/** File or folder compress size */
	wstring	   m_strModifyTime;		/** File or folder modification time */
	wstring	   m_strName;			/** File or folder full path */
};

/*!
 * \brief
 * Domain IP list of QNAP Turbo NAS
 * 
 */
class QTSHTTPDLL_API CQtsHttpNASInfo
{
public:
	/** Default constructor */
	CQtsHttpNASInfo();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] strMyCloudNasName : My cloud NAS name.
	 * \param 
	 * [IN] strNasExternalIP : External IP of QNAP Turbo NAS.
	 * \param 
	 * [IN] strNasLocalIP : Local IP of QNAP Turbo NAS.
	 * \param 
	 * [IN] strNasHostIP : Host IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpNASInfo( 
		LPCTSTR	   strMyCloudNasName,
		LPCTSTR	   strNasExternalIP,
		LPCTSTR	   strNasLocalIP,
		LPCTSTR	   strNasHostIP
		);

	/*!
	 * \brief
	 * Get my cloud NAS name.
	 * 
	 * \returns
	 * LPCTSTR : My cloud NAS name.
	 */
	LPCTSTR	GetMyCloudNasName();
	/*!
	 * \brief
	 * Get external IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * LPCTSTR : External IP of QNAP Turbo NAS.
	 */
	LPCTSTR	GetNasExternalIP();
	/*!
	 * \brief
	 * Get local IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * LPCTSTR : Local IP of QNAP Turbo NAS.
	 */
	LPCTSTR	GetNasLocalIP();
	/*!
	 * \brief
	 * Get host IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * LPCTSTR : Host IP of QNAP Turbo NAS.
	 */
	LPCTSTR	GetNasHostIP();

	/*!
	 * \brief
	 * Set my cloud NAS name.
	 * 
	 * \param
	 * [IN] strMyCloudNasName : My cloud NAS name.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetMyCloudNasName(LPCTSTR strMyCloudNasName);
	/*!
	 * \brief
	 * Set external IP of QNAP Turbo NAS.
	 * 
	 * \param
	 * [IN] strNasExternalIP : External IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetNasExternalIP(LPCTSTR strNasExternalIP);
	/*!
	 * \brief
	 * Set local IP of QNAP Turbo NAS.
	 * 
	 * \param
	 * [IN] strNasLocalIP : Local IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetNasLocalIP(LPCTSTR strNasLocalIP);
	/*!
	 * \brief
	 * Set host IP of QNAP Turbo NAS.
	 * 
	 * \param
	 * [IN] strNasHostIP : Host IP of QNAP Turbo NAS.
	 * 
	 * \returns
	 * n/a.
	 */
	void SetNasHostIP(LPCTSTR strNasHostIP);

private:
	wstring	   m_strMyCloudNasName;		/** My cloud NAS name */
	wstring	   m_strNasExternalIP;		/** External IP of QNAP Turbo NAS */
	wstring	   m_strNasLocalIP;			/** Local IP of QNAP Turbo NAS */
	wstring	   m_strNasHostIP;			/** Host IP of QNAP Turbo NAS */
};

/*!
 * \brief
 * Interface for file station request base
 *  
 * 1. One file station API map to one file station request.
 * 2. Abort FS request.
 * 3. Get response from file station API.
 */
class IQtsHttpFileStationRequestBase
{
public:
	/*!
	 * \brief
	 * Developer can abort file station API by calling Abort() anytime.
	 * 
	 * \returns
	 * TRUE : Abort success.
	 * \returns
	 * FALSE : Abort fail.
	 */
	virtual BOOL Abort() = 0;

	/*!
	 * \brief
	 * Get file list count after GetFileListCountAsync API success.
	 * 
	 * \returns
	 * int : Output file list count.
	 */
	virtual int ResponseFileListCount() = 0;

	/*!
	 * \brief
	 * Get file list after GetShareFolderListAsync/GetFileListAsync/SearchFileAsync API success.
	 * 
	 * \returns
	 * vector<CQtsHttpFileEntry>* : Output file list.
	 */
	virtual vector<CQtsHttpFileEntry>* ResponseFileList() = 0;

	/*!
	 * \brief
	 * Get extract file list after ListContentsOfExtractedFileAsync API success.
	 * 
	 * \returns
	 * vector<CQtsHttpExtractEntry>* : Output extract file list.
	 */
	virtual vector<CQtsHttpExtractEntry>* ResponseExtractFileList() = 0;

	/*!
	 * \brief
	 * Get share link after CreateShareLinkAsync API success.
	 * 
	 * \returns
	 * CQtsHttpShareLink : Output share link.
	 */
	virtual CQtsHttpShareLink ResponseShareLink() = 0;

	/*!
	 * \brief
	 * Get share link list after GetShareLinkListAsync API success.
	 * 
	 * \returns
	 * vector<CQtsHttpShareLink>* : Output share link list.
	 */
	virtual vector<CQtsHttpShareLink>* ResponseShareLinkList() = 0;

	/*!
	 * \brief
	 * Get file/folder status after GetFileStatusAsync API success.
	 * 
	 * \returns
	 * CQtsHttpFileStatus : Output file/folder status.
	 */
	virtual CQtsHttpFileStatus ResponseFileStatus() = 0;

	/*!
	 * \brief
	 * Get hostname and external IP address of the QNAP Turbo NAS after GetDomainIPListAsync API success.
	 * 
	 * \returns
	 * CQtsHttpNASInfo : Output hostname and external IP address.
	 */
	virtual CQtsHttpNASInfo ResponseNASInfo() = 0;

	/*!
	 * \brief
	 * Get chunked upload ID after CreateChunkedUploadIDAsync API success.
	 * 
	 * \returns
	 * LPCTSTR : Output chunked upload ID.
	 */
	virtual LPCTSTR ResponseChunkedUploadID() = 0;
};

/*!
 * \brief
 * Interface for Qts Http file station API
 *  
 * 1. Support file station related API.
 * 2. Support sync and async API.
 */
class IQtsHttpFileStation
{
public:
	/*!
	 * \brief
	 * Get share folder list of QNAP Turbo NAS.
	 * 
	 * \param
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] ShareFolderList : Output list pointer of share folder.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  GetShareFolderList(BOOL* pbCancel, vector<CQtsHttpFileEntry>* ShareFolderList) = 0;

	/*!
	 * \brief
	 * Get file list count below specify path.
	 * 
	 * \param 
	 * [IN] strPath : Specify folder path.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] nTotalFileCount : Output file list count.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 *
	 * \remarks
	 * File list count include files and folders.
	 */
	virtual QTS_HTTP_API_RESULT  GetFileListCount(LPCTSTR strPath, BOOL* pbCancel, int &nTotalFileCount) = 0;

	/*!
	 * \brief
	 * Get file list below specify path.
	 * 
	 * \param 
	 * [IN] strPath : Specify folder path.
	 * \param 
	 * [IN] nStartIdx : Start index of get files.
	 * \param 
	 * [IN] nFileLimit : Number of get files.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] FileList : Output file list.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 *
	 * \remarks
	 * File list include files and folders.
	 */
	virtual	QTS_HTTP_API_RESULT  GetFileList(LPCTSTR strPath, int nStartIdx, int nFileLimit, BOOL* pbCancel, vector<CQtsHttpFileEntry>* FileList) = 0;

	/*!
	 * \brief
	 * Get file URL.
	 * 
	 * \param 
	 * [IN] strFileFullPath : File full path.
	 * \param 
	 * [OUT] strFileURL : Output URL of file.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  GetFileURL(LPCTSTR strFileFullPath, LPTSTR strFileURL) = 0;

	/*!
	 * \brief
	 * Get image thumbnail of file.
	 * 
	 * \param 
	 * [IN] strFileFullPath : File full path.
	 * \param 
	 * [OUT] strFileThumbURL : Output URL of image thumbnail file.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  GetImgThumbURL(LPCTSTR strFileFullPath, LPTSTR strFileThumbURL) = 0;
	
	/*!
	 * \brief
	 * Upload a file to QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strUploadFileFullPath : Local path of upload file.
	 * \param 
	 * [IN] strToFilePath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API TotalBytesTransferred status.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  UploadFileByPath(LPCTSTR strUploadFileFullPath, LPCTSTR strToFilePath, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * Create chunked upload ID.
	 * 
	 * \param 
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] strUploadID : Output chunked upload ID.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CreateChunkedUploadID(LPCTSTR strUploadTempPath, BOOL* pbCancel, LPTSTR strUploadID) = 0;

	/*!
	 * \brief
	 * Chunked upload a file to QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strUploadFileFullPath : Local path of chunked upload file.
	 * \param 
	 * [IN] strToFilePath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strRenameOriDestFileName : Rename original destination file to strRenameOriDestFileName, if file already exist.
	 * \param 
	 * [IN] strUploadID : Chunked upload ID. Get from CreateChunkedUploadID.
	 * \param 
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API TotalBytesTransferred status.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  ChunkedUploadFileByPath(LPCTSTR strUploadFileFullPath, LPCTSTR strToFilePath, LPCTSTR strRenameOriDestFileName, LPCTSTR strUploadID, LPCTSTR strUploadTempPath, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfHttpFSCallback = NULL) = 0;
	
	/*!
	 * \brief
	 * Delete chunked upload file by upload ID.
	 * 
	 * \param 
	 * [IN] strUploadID : Chunked upload ID. Get from CreateChunkedUploadID.
	 * \param 
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DeleteChunkedUploadFile(LPCTSTR strUploadID, LPCTSTR strUploadTempPath, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Download a file to local.
	 * 
	 * \param 
	 * [IN] downloadFileEntry : Download file entry in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToFilePath : Destination path in local.
	 * \param 
	 * [IN] llOffset : Resume download from offset.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API TotalBytesTransferred status.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DownloadFile(CQtsHttpFileEntry downloadFileEntry, LPCTSTR strToFilePath, LONGLONG llOffset, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfHttpFSCallback = NULL) = 0;
  
	/*!
	 * \brief
	 * Download a file by path to local.
	 * 
	 * \param 
	 * [IN] strDownloadFileFullPath : Download file full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToFilePath : Destination path in local.
	 * \param 
	 * [IN] llOffset : Resume download from offset.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API TotalBytesTransferred status.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DownloadFileByPath(LPCTSTR strDownloadFileFullPath, LPCTSTR strToFilePath, LONGLONG llOffset, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfHttpFSCallback = NULL) = 0;
  
	/*!
	 * \brief
	 * Create share link.
	 * 
	 * \param 
	 * [IN] ShareLinkInfo : Setting of create share link.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] ShareLink : Output share link information.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
  	virtual	QTS_HTTP_API_RESULT  CreateShareLink(CQtsHttpShareLinkInfo ShareLinkInfo, BOOL* pbCancel, CQtsHttpShareLink &ShareLink) = 0;

	/*!
	 * \brief
	 * Get share link list.
	 * 
	 * \param 
	 * [IN] nFileLimit : Number of share link list.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] ShareLinkList : Output share link list on QNAP Turbo NAS.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
  	virtual	QTS_HTTP_API_RESULT  GetShareLinkList(int nFileLimit, BOOL* pbCancel, vector<CQtsHttpShareLink>* ShareLinkList) = 0;

	/*!
	 * \brief
	 * Delete share link.
	 * 
	 * \param 
	 * [IN] ShareLink : Share link on QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
  	virtual	QTS_HTTP_API_RESULT  DeleteShareLink(CQtsHttpShareLink ShareLink, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Create a folder by path.
	 * 
	 * \param 
	 * [IN] strCreateFolderPath : Create a folder at this path.
	 * \param 
	 * [IN] strCreateFolderName : Create folder name.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CreateFolderByPath(LPCTSTR strCreateFolderPath, LPCTSTR strCreateFolderName, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Create a folder.
	 * 
	 * \param 
	 * [IN] parentFolderEntry : Create a folder below parentFolderEntry.
	 * \param 
	 * [IN] strCreateFolderName : Create folder name.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CreateFolder(CQtsHttpFileEntry parentFolderEntry, LPCTSTR strCreateFolderName, BOOL* pbCancel) = 0;
	
	/*!
	 * \brief
	 * Copy a file/folder by path from source to destination.
	 * 
	 * \param 
	 * [IN] strFromPath : Source full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CopyFileByPath(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * Copy a file/folder from source to destination.
	 * 
	 * \param 
	 * [IN] fromEntry : Source entry in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CopyFile(CQtsHttpFileEntry fromEntry, LPCTSTR strToPath, int nMode, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;


	/*!
	 * \brief
	 * Move a file/folder by path from source to destination.
	 * 
	 * \param 
	 * [IN] strFromPath : Source full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  MoveFileByPath(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * Move a file/folder from source to destination.
	 * 
	 * \param 
	 * [IN] fromEntry : Source entry in QNAP Turbo NAS.
	 * \param 
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param 
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  MoveFile(CQtsHttpFileEntry fromEntry, LPCTSTR strToPath, int nMode, BOOL* pbCancel, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * Delete a file/folder by path in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strDeletePath : Delete full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DeleteFileByPath(LPCTSTR strDeletePath, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Delete a file/folder in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] deleteFileEntry : Delete entry in QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DeleteFile(CQtsHttpFileEntry deleteFileEntry, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Rename a file/folder in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strRenamePath : Rename full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strNewName : New folder/file name.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  RenameFile(LPCTSTR strRenamePath, LPCTSTR strNewName, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Set folder/file modification time.
	 * 
	 * \param 
	 * [IN] strUpdateFilePath : Update file full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] modifyTime : Epoch time (seconds since 1970-01-01 00:00:00 UTC).
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] FileStat : Output file status of modify time file/folder.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  UpdateFileModifyDate(LPCTSTR strUpdateFilePath, long modifyTime, BOOL* pbCancel, CQtsHttpFileStatus &FileStat) = 0;

	/*!
	 * \brief
	 * File search.
	 * 
	 * \param 
	 * [IN] strPath :Search file path in QNAP Turbo NAS.
	 * \param 
	 * [IN] strQuery : Query keyword.
	 * \param 
	 * [IN] nFileLimit : Number of response file list.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] FileList : Output list of files.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
  	virtual	QTS_HTTP_API_RESULT  SearchFile(LPCTSTR strPath, LPCTSTR strQuery, int nFileLimit, BOOL* pbCancel, vector<CQtsHttpFileEntry>* FileList) = 0;

	/*!
	 * \brief
	 * Get file status.
	 * 
	 * \param 
	 * [IN] strPath :File full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] FileStat : Output file status.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
 	virtual	QTS_HTTP_API_RESULT  GetFileStatus(LPCTSTR strPath, BOOL* pbCancel, CQtsHttpFileStatus &FileStat) = 0;

	/*!
	 * \brief
	 * Compress files.
	 * 
	 * \param 
	 * [IN] strCompressName :Compressed name.
	 * \param 
	 * [IN] compressFilePath : Compressd file path.
	 * \param 
	 * [IN] totalFileNum : The amount of compression files number.
	 * \param 
	 * [IN] compressFileNameList : The compressed file name list.
	 * \param 
	 * [IN] compressFormat : Compressed format.
	 * \param 
	 * [IN] strPWD : Compressed password (can be null).
	 * \param 
	 * [IN] compressLevel : Compressed level.
	 * \param 
	 * [IN] encryptType : Compressed encrypt type.
	 * \param 
	 * [IN] updateMode : Compressed update mode.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  CompressFiles(LPCTSTR strCompressName, LPCTSTR compressFilePath, int totalFileNum, vector<LPCTSTR>* compressFileNameList, QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE compressFormat = QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_ZIP, LPCTSTR strPWD = L"", \
		QTS_HTTP_FILE_STATION_COMPRESS_LEVEL compressLevel = QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_NORMAL, QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE encryptType = QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_AES256, \
		QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE updateMode = QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_ADD_REPLACE, BOOL* pbCancel = NULL, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * List the contents of an extracted file.
	 * 
	 * \param 
	 * [IN] strExtractedFileFullName :Extracted file full path in QTS Turbo NAS.
	 * \param 
	 * [IN] nStartIdx : Response files start index.
	 * \param 
	 * [IN] nFileLimit : Number of response files.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] ExtractedFileList : Output extracted file list.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  ListContentsOfExtractedFile(LPCTSTR strExtractedFileFullName, int nStartIdx, int nFileLimit, BOOL* pbCancel, vector<CQtsHttpExtractEntry>* ExtractedFileList) = 0;

	/*!
	 * \brief
	 * Extract files.
	 * 
	 * \param 
	 * [IN] strExtractedFileFullName : Extracted file full path in QTS Turbo NAS.
	 * \param 
	 * [IN] strToPath : Path of the extracted files.
	 * \param 
	 * [IN] strPWD : Extraction password (can be null).
	 * \param 
	 * [IN] extractMode : Extraction mode.
	 * \param 
	 * [IN] partTotalFileSize : The total number of extracted files, if extractMode = QTS_EXTRACT_MODE_PART.
	 * \param 
	 * [IN] partExtractfileNameList : Name of the file to be extracted (can be more than one), if extractMode = QTS_EXTRACT_MODE_PART.
	 * \param 
	 * [IN] overWrite : Skip / Overwrite.
	 * \param 
	 * [IN] extractPathMode : Extracted path mode.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param 
	 * [IN] pfHttpFSCallback : Callback function for notify API progress rate.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  ExtractFiles(LPCTSTR strExtractedFileFullName, LPCTSTR strToPath, LPCTSTR strPWD, QTS_HTTP_FILE_STATION_EXTRACT_MODE extractMode, int partTotalFileSize, vector<LPCTSTR>* partExtractfileNameList, QTS_HTTP_FILE_STATION_OVERWRITE overWrite = QTS_HTTP_FILE_STATION_OVERWRITE_OVERWRITE, \
		QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE extractPathMode = QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_FULL, BOOL* pbCancel = NULL, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * Get hostname and external IP address of the QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param 
	 * [OUT] NASInfo : Output hostname and external IP address.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  GetDomainIPList(BOOL* pbCancel, CQtsHttpNASInfo &NASInfo) = 0;

	/*!
	 * \brief
	 * Add video to transcode queue.
	 * 
	 * \param 
	 * [IN] strVideoFileFullPath : Add video file full path in QNAP Turbo NAS.
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  AddVideoTranscode(LPCTSTR strVideoFileFullPath, BOOL* pbCancel) = 0;

	/*!
	 * \brief
	 * Delete video transcode file.
	 * 
	 * \param 
	 * [IN] strVideoFileFullPath : Delete video file full path in QNAP Turbo NAS.
	 * \param
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  DeleteVideoTranscode(LPCTSTR strVideoFileFullPath, BOOL* pbCancel) = 0;

	/*!
	* \brief
	* Destroy file station request after file station API complete.
	* 
	* \param 
	* [IN] request : File station request.
	* 
	* \returns
	* TRUE : Destroy file station request success.
	* \returns
	* FALSE : Destroy file station request fail.
	*/
	virtual	BOOL DestroyRequest(IQtsHttpFileStationRequestBase** request) = 0;

	/*!
	 * \brief
	 * [Async] Get share folder list of QNAP Turbo NAS.
	 * 
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  GetShareFolderListAsync(LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Get file list count below specify path.
	 * 
	 * \param 
	 * [IN] strPath : Specify folder path.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual IQtsHttpFileStationRequestBase*  GetFileListCountAsync(LPCTSTR strPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Get file list below specify path.
	 * 
	 * \param 
	 * [IN] strPath : Specify folder path.
	 * \param
	 * [IN] nStartIdx : Start index of get files.
	 * \param
	 * [IN] nFileLimit : Number of get files.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual IQtsHttpFileStationRequestBase*  GetFileListAsync(LPCTSTR strPath, int nStartIdx, int nFileLimit, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;
	
	/*!
	 * \brief
	 * [Async] Create share link.
	 * 
	 * \param 
	 * [IN] ShareLinkInfo : Setting of create share link.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CreateShareLinkAsync(CQtsHttpShareLinkInfo ShareLinkInfo, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Get share link list.
	 * 
	 * \param 
	 * [IN] nFileLimit : Number of share link list.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
  	virtual	IQtsHttpFileStationRequestBase*  GetShareLinkListAsync(int nFileLimit, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Delete share link.
	 * 
	 * \param 
	 * [IN] ShareLink : Share link on QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
  	virtual	IQtsHttpFileStationRequestBase*  DeleteShareLinkAsync(CQtsHttpShareLink ShareLink, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;
	
	/*!
	 * \brief
	 * [Async] Upload a file by path to QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strUploadFileFullPath : Local path of upload file.
	 * \param
	 * [IN] strToFilePath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  UploadFileByPathAsync(LPCTSTR strUploadFileFullPath, LPCTSTR strToFilePath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Create chunked upload ID.
	 * 
	 * \param 
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CreateChunkedUploadIDAsync(LPCTSTR strUploadTempPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Chunked upload a file to QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strUploadFileFullPath : Local path of chunked upload file.
	 * \param
	 * [IN] strToFilePath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] strRenameOriDestFileName : Rename original destination file to strRenameOriDestFileName, if file already exist.
	 * \param
	 * [IN] strUploadID : Chunked upload ID. Get from CreateChunkedUploadID.
	 * \param
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  ChunkedUploadFileByPathAsync(LPCTSTR strUploadFileFullPath, LPCTSTR strToFilePath, LPCTSTR strRenameOriDestFileName, LPCTSTR strUploadID, LPCTSTR strUploadTempPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Delete chunked upload file by upload ID.
	 * 
	 * \param 
	 * [IN] strUploadID : Chunked upload ID. Get from CreateChunkedUploadID.
	 * \param
	 * [IN] strUploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DeleteChunkedUploadFileAsync(LPCTSTR strUploadID, LPCTSTR strUploadTempPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Download a file by path to local.
	 * 
	 * \param 
	 * [IN] strDownloadFileFullPath : Download file full path in QNAP Turbo NAS.
	 * \param
	 * [IN] strToFilePath : Destination path in local.
	 * \param 
	 * [IN] llOffset : Resume download from offset.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DownloadFileByPathAsync(LPCTSTR strDownloadFileFullPath, LPCTSTR strToFilePath, LONGLONG llOffset, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Download a file to local.
	 * 
	 * \param 
	 * [IN] downloadFileEntry : Download file entry in QNAP Turbo NAS.
	 * \param
	 * [IN] strToFilePath : Destination path in local.
	 * \param 
	 * [IN] llOffset : Resume download from offset.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DownloadFileAsync(CQtsHttpFileEntry downloadFileEntry, LPCTSTR strToFilePath, LONGLONG llOffset, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Create a folder by path.
	 * 
	 * \param 
	 * [IN] strCreateFolderPath : Create a folder at this path.
	 * \param
	 * [IN] strCreateFolderName : Create folder name.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CreateFolderByPathAsync(LPCTSTR strCreateFolderPath, LPCTSTR strCreateFolderName, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Create a folder.
	 * 
	 * \param 
	 * [IN] parentFolderEntry : Create a folder below parentFolderEntry.
	 * \param
	 * [IN] strCreateFolderName : Create folder name.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CreateFolderAsync(CQtsHttpFileEntry parentFolderEntry, LPCTSTR strCreateFolderName, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;


	/*!
	 * \brief
	 * [Async] Copy a file/folder by path from source to destination.
	 * 
	 * \param 
	 * [IN] strFromPath : Source full path in QNAP Turbo NAS.
	 * \param
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CopyFileByPathAsync(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Copy a file/folder from source to destination.
	 * 
	 * \param 
	 * [IN] fromEntry : Source entry in QNAP Turbo NAS.
	 * \param
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CopyFileAsync(CQtsHttpFileEntry fromEntry, LPCTSTR strToPath, int nMode, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Move a file/folder by path from source to destination.
	 * 
	 * \param 
	 * [IN] strFromPath : Source full path in QNAP Turbo NAS.
	 * \param
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  MoveFileByPathAsync(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Move a file/folder from source to destination.
	 * 
	 * \param 
	 * [IN] fromEntry : Source entry in QNAP Turbo NAS.
	 * \param
	 * [IN] strToPath : Destination path in QNAP Turbo NAS.
	 * \param
	 * [IN] nMode : Skip (1) / Overwrite (0).
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  MoveFileAsync(CQtsHttpFileEntry fromEntry, LPCTSTR strToPath, int nMode, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Delete a file/folder by path in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strDeletePath : Delete full path in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DeleteFileByPathAsync(LPCTSTR strDeletePath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Delete a file/folder in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] deleteFileEntry : Delete entry in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DeleteFileAsync(CQtsHttpFileEntry deleteFileEntry, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Rename a file/folder in QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] strRenamePath : Rename full path in QNAP Turbo NAS.
	 * \param
	 * [IN] strNewName : New folder/file name.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  RenameFileAsync(LPCTSTR strRenamePath, LPCTSTR strNewName, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Set folder/file modification time.
	 * 
	 * \param 
	 * [IN] strUpdateFilePath : Update file full path in QNAP Turbo NAS.
	 * \param
	 * [IN] modifyTime : Epoch time (seconds since 1970-01-01 00:00:00 UTC).
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  UpdateFileModifyDateAsync(LPCTSTR strUpdateFilePath, long modifyTime, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] File search.
	 * 
	 * \param 
	 * [IN] strPath :Search file path in QNAP Turbo NAS.
	 * \param
	 * [IN] strQuery : Query keyword.
	 * \param
	 * [IN] nFileLimit : Number of response file list.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  SearchFileAsync(LPCTSTR strPath, LPCTSTR strQuery, int nFileLimit, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Get file status.
	 * 
	 * \param 
	 * [IN] strPath :File full path in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
 	virtual	IQtsHttpFileStationRequestBase*  GetFileStatusAsync(LPCTSTR strPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Compress files.
	 * 
	 * \param 
	 * [IN] strCompressName :Compressed name.
	 * \param
	 * [IN] compressFilePath : Compressd file path.
	 * \param
	 * [IN] totalFileNum : The amount of compression files number.
	 * \param
	 * [IN] compressFileNameList : The compressed file name list.
	 * \param
	 * [IN] compressFormat : Compressed format.
	 * \param
	 * [IN] strPWD : Compressed password (can be null).
	 * \param
	 * [IN] compressLevel : Compressed level.
	 * \param
	 * [IN] encryptType : Compressed encrypt type.
	 * \param
	 * [IN] updateMode : Compressed update mode.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  CompressFilesAsync(LPCTSTR strCompressName, LPCTSTR compressFilePath, int totalFileNum, vector<LPCTSTR>* compressFileNameList, QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE compressFormat = QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_ZIP, LPCTSTR strPWD = L"", \
												QTS_HTTP_FILE_STATION_COMPRESS_LEVEL compressLevel = QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_NORMAL, QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE encryptType = QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_AES256, \
												QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE updateMode = QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_ADD_REPLACE, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;
	
	/*!
	 * \brief
	 * [Async] List the contents of an extracted file.
	 * 
	 * \param 
	 * [IN] strExtractedFileFullName :Extracted file full path in QTS Turbo NAS.
	 * \param
	 * [IN] nStartIdx : Response files start index.
	 * \param
	 * [IN] nFileLimit : Number of response files.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  ListContentsOfExtractedFileAsync(LPCTSTR strExtractedFileFullName, int nStartIdx, int nFileLimit, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Extract files.
	 * 
	 * \param 
	 * [IN] strExtractedFileFullName : Extracted file full path in QTS Turbo NAS.
	 * \param
	 * [IN] strToPath : Path of the extracted files.
	 * \param
	 * [IN] strPWD : Extraction password (can be null).
	 * \param
	 * [IN] extractMode : Extraction mode.
	 * \param
	 * [IN] partTotalFileSize : The total number of extracted files, if extractMode = QTS_EXTRACT_MODE_PART.
	 * \param
	 * [IN] partExtractfileNameList : Name of the file to be extracted (can be more than one), if extractMode = QTS_EXTRACT_MODE_PART.
	 * \param
	 * [IN] overWrite : Skip / Overwrite.
	 * \param
	 * [IN] extractPathMode : Extracted path mode.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  ExtractFilesAsync(LPCTSTR strExtractedFileFullName, LPCTSTR strToPath, LPCTSTR strPWD, QTS_HTTP_FILE_STATION_EXTRACT_MODE extractMode, int partTotalFileSize, vector<LPCTSTR>* partExtractfileNameList, QTS_HTTP_FILE_STATION_OVERWRITE overWrite = QTS_HTTP_FILE_STATION_OVERWRITE_OVERWRITE, \
											   QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE extractPathMode = QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_FULL, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;
	/*!
	 * \brief
	 * [Async] Get hostname and external IP address of the QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  GetDomainIPListAsync(LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Add video to transcode queue.
	 * 
	 * \param 
	 * [IN] strVideoFileFullPath : Add video file full path in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  AddVideoTranscodeAsync(LPCTSTR strVideoFileFullPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Delete video transcode file.
	 * 
	 * \param 
	 * [IN] strVideoFileFullPath : Delete video file full path in QNAP Turbo NAS.
	 * \param
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpFSCallback : File station async callback function.
	 * 
	 * \returns
	 * IQtsHttpFileStationRequestBase* : File station request pointer object.
	 */
	virtual	IQtsHttpFileStationRequestBase*  DeleteVideoTranscodeAsync(LPCTSTR strVideoFileFullPath, LPVOID lpData = NULL, PFQTS_HTTP_FILE_STATION_ASYNC_RESPONSE_CALLBACK pfHttpFSCallback = NULL) = 0;
};

#endif	// QTS_HTTP_FILE_STATIONS_H
