#include "StdAfx.h"
#include <iostream>
#include <fstream>
#include "QTSHAPIWrapper.h"

QTSHAPIWrapper::QTSHAPIWrapper(void)
{
	m_pQTSHTTPFileStation = NULL;
	m_pQTSHTTPServer = NULL;
	m_FSPort = 0;
	m_FSSSLPort = 0;
}

QTSHAPIWrapper::~QTSHAPIWrapper(void)
{
	if (m_pQTSHTTPServer != NULL)
	{
		delete m_pQTSHTTPServer;
		m_pQTSHTTPServer = NULL;
	}
}

void QTSHAPIWrapper::SetServerInfo(CQtsHttpServerInfo serverInfo)
{
	m_serverInfo = serverInfo;
}

void QTSHAPIWrapper::SetFSPort(long port)
{
	m_FSPort = port;
}

void QTSHAPIWrapper::SetFSSSLPort(long SSLport)
{
	m_FSSSLPort = SSLport;
}

IQtsHttpFileStation* QTSHAPIWrapper::OpenFileStation()
{
	QTS_HTTP_API_RESULT	t_Result = QTS_HTTP_API_FAILURE;
	if (m_pQTSHTTPServer == NULL)
		m_pQTSHTTPServer = new CQtsHttpServer(m_serverInfo);

	if (m_serverInfo.IsSecureMode())
		m_pQTSHTTPServer->SetFileStationSSLPortNum(m_FSSSLPort);
	else
		m_pQTSHTTPServer->SetFileStationPortNum(m_FSPort);

	m_pQTSHTTPServer->SetTimeouts(60000);
	m_pQTSHTTPServer->SetAgentName(L"QNAPSample");

	t_Result = m_pQTSHTTPServer->Login(QTS_HTTP_STATION_TYPE_FILE_STATION);
	m_pQTSHTTPFileStation = NULL;
	if (t_Result == QTS_HTTP_API_SUCCESS && m_pQTSHTTPServer->QueryFileStationVersion() == QTS_HTTP_FILE_STATION_API_V1)
	{
		m_pQTSHTTPFileStation = m_pQTSHTTPServer->OpenFileStation();
	}
	else
	{
		DWORD dwError = GetLastError();
		printf("System Error = %d\n", dwError);
	}
	return m_pQTSHTTPFileStation;
}

void QTSHAPIWrapper::CloseFileStation()
{
	if (m_pQTSHTTPServer != NULL)
	{
		if (m_pQTSHTTPFileStation != NULL)
			m_pQTSHTTPServer->CloseFileStation(&m_pQTSHTTPFileStation);
		delete m_pQTSHTTPServer;
		m_pQTSHTTPServer = NULL;
	}
}

BOOL QTSHAPIWrapper::GetFileListWithDetailInfo(LPCTSTR strPath, int nStartIdx, int nFileLimit, BOOL* pbCancel, std::vector<CQtsHttpFileStatus>* FileList)
{
	BOOL result = FALSE;
	int t_nFileCount = 0;
	if (m_pQTSHTTPFileStation->GetFileListCount(strPath, pbCancel, t_nFileCount) == QTS_HTTP_API_SUCCESS && t_nFileCount > nStartIdx && pbCancel != NULL && *pbCancel == FALSE)
	{
		std::vector<CQtsHttpFileEntry> vFolderList;
		if (m_pQTSHTTPFileStation->GetFileList(strPath, nStartIdx, nFileLimit, pbCancel, &vFolderList) == QTS_HTTP_API_SUCCESS && pbCancel != NULL && *pbCancel == FALSE)
		{
			for (size_t i = 0; i < vFolderList.size(); i++)
			{
				CQtsHttpFileStatus t_FileStatus;
				std::wstring strPath = vFolderList.data()[i].GetFilePath();
				std::wstring strName = vFolderList.data()[i].GetFileName();
				std::wstring t_strFileFullPath = strPath;
				t_strFileFullPath.append(L"/");
				t_strFileFullPath += strName;
				if (pbCancel != NULL && *pbCancel == FALSE && m_pQTSHTTPFileStation->GetFileStatus(t_strFileFullPath.c_str(), pbCancel, t_FileStatus) == QTS_HTTP_API_SUCCESS)
				{
					FileList->push_back(t_FileStatus);
				}
			}
			if (pbCancel != NULL && *pbCancel == FALSE)
				result = TRUE;
		}
	}

	return result;
}

BOOL QTSHAPIWrapper::UploadFileWithResume(BOOL bResume, LPCTSTR strUploadFileFullPath, LPCTSTR strToFilePath, LPCTSTR strRenameOriDestFileName, BOOL* pbCancel, LPVOID lpData, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfUICallback)
{
	BOOL result = FALSE;
	TCHAR line[1024];
	TCHAR uploadID[256];
	TCHAR t_ChunkUploadID[256];
	BOOL bCanResume = FALSE;
	std::wfstream fin;
	fin.open(L"ChunkedUploadIDList.txt", std::ios::in);

	while (fin.getline(line, 1024, '\n'))
	{
		if (wcscmp(line, strUploadFileFullPath) == 0)
		{
			fin.getline(uploadID, 256, '\n');
			bCanResume = TRUE;
			break;
		}
	}
	fin.close();
	fin.clear();

	if (bResume && bCanResume)		// Need resume upload
	{
		if (m_pQTSHTTPFileStation->ChunkedUploadFileByPath(strUploadFileFullPath, strToFilePath, strRenameOriDestFileName, uploadID, L"/Public", pbCancel, lpData, pfUICallback) == QTS_HTTP_API_SUCCESS)
		{
			// delete resume data
			std::wfstream tmpfin;
			std::wfstream tmpfout;
			TCHAR tmpline[1024];
			tmpfin.open(L"ChunkedUploadIDList.txt", std::ios::in);
			tmpfout.open(L"tmp.txt", std::ios::out);

			while (tmpfin.getline(tmpline, 1024, '\n'))
			{
				if (wcscmp(tmpline, strUploadFileFullPath) == 0)
				{
					tmpfin.getline(tmpline, 1024, '\n');
					continue;
				}
				tmpfout << tmpline << std::endl;
			}
			tmpfin.close();
			tmpfin.clear();
			tmpfout.close();
			tmpfout.clear();

			std::wfstream t_file;
			t_file.open(L"ChunkedUploadIDList.txt", std::ios::out | std::ios::trunc | std::ios::binary);
			std::wfstream iftmp;
			iftmp.open(L"tmp.txt", std::ios::in | std::ios::binary);
			while(iftmp.getline(tmpline, 1024, '\n'))
			{
				t_file << tmpline << std::endl;
			}
			iftmp.close();
			iftmp.clear();
			t_file.close();
			t_file.clear();
			remove("tmp.txt");

			result = TRUE;
		}
	}
	else
	{
		if (bCanResume)
		{
			// delete resume data
			std::wfstream tmpfin;
			std::wfstream tmpfout;
			TCHAR tmpline[1024];
			TCHAR t_DelChunkUploadID[256];
			tmpfin.open(L"ChunkedUploadIDList.txt", std::ios::in);
			tmpfout.open(L"tmp.txt", std::ios::out);

			while (tmpfin.getline(tmpline, 1024, '\n'))
			{
				if (wcscmp(tmpline, strUploadFileFullPath) == 0)
				{
					tmpfin.getline(t_DelChunkUploadID, 1024, '\n');
					continue;
				}
				tmpfout << tmpline << std::endl;
			}
			tmpfin.close();
			tmpfin.clear();
			tmpfout.close();
			tmpfout.clear();

			std::wfstream t_file;
			t_file.open(L"ChunkedUploadIDList.txt", std::ios::out | std::ios::trunc | std::ios::binary);
			std::wfstream iftmp;
			iftmp.open(L"tmp.txt", std::ios::in | std::ios::binary);
			while(iftmp.getline(tmpline, 1024, '\n'))
			{
				t_file << tmpline << std::endl;
			}
			iftmp.close();
			iftmp.clear();
			t_file.close();
			t_file.clear();
			remove("tmp.txt");
			m_pQTSHTTPFileStation->DeleteChunkedUploadFile(t_DelChunkUploadID, L"/Public", pbCancel);
		}

		if (m_pQTSHTTPFileStation->CreateChunkedUploadID(L"/Public", pbCancel, t_ChunkUploadID) == QTS_HTTP_API_SUCCESS)
		{
			std::wfstream fout;
			fout.open(L"ChunkedUploadIDList.txt", std::ios::out | std::ios::app);
			fout << strUploadFileFullPath << std::endl;
			fout << t_ChunkUploadID << std::endl;
			fout.close();
			fout.clear();

			if (m_pQTSHTTPFileStation->ChunkedUploadFileByPath(strUploadFileFullPath, strToFilePath, strRenameOriDestFileName, t_ChunkUploadID, L"/Public", pbCancel, lpData, pfUICallback) == QTS_HTTP_API_SUCCESS)
			{
				// delete resume data
				std::wfstream tmpfin;
				std::wfstream tmpfout;
				TCHAR tmpline[1024];
				tmpfin.open(L"ChunkedUploadIDList.txt", std::ios::in);
				tmpfout.open(L"tmp.txt", std::ios::out);

				while (tmpfin.getline(tmpline, 1024, '\n'))
				{
					if (wcscmp(tmpline, strUploadFileFullPath) == 0)
					{
						tmpfin.getline(tmpline, 1024, '\n');
						continue;
					}
					tmpfout << tmpline << std::endl;
				}
				tmpfin.close();
				tmpfin.clear();
				tmpfout.close();
				tmpfout.clear();

				std::wfstream t_file;
				t_file.open(L"ChunkedUploadIDList.txt", std::ios::out | std::ios::trunc | std::ios::binary);
				std::wfstream iftmp;
				iftmp.open(L"tmp.txt", std::ios::in | std::ios::binary);
				while(iftmp.getline(tmpline, 1024, '\n'))
				{
					t_file << tmpline << std::endl;
				}
				iftmp.close();
				iftmp.clear();
				t_file.close();
				t_file.clear();
				remove("tmp.txt");
				result = TRUE;
			}
		}
	}

	return result;
}

BOOL QTSHAPIWrapper::DownloadFileWithResume(BOOL bResume, LPCTSTR strDownloadFileFullPath, LPCTSTR strToFilePath, BOOL* pbCancel, LPVOID lpData, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSBYTES_CALLBACK pfHttpFSCallback)
{
	BOOL result = FALSE;
	HANDLE hFile = NULL;
	LONGLONG t_FileOffset = 0;

	hFile = ::CreateFile(strToFilePath, 
		GENERIC_WRITE, 
		FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, 
		NULL, 
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_TEMPORARY | FILE_FLAG_BACKUP_SEMANTICS, 
		NULL);

	if(hFile == NULL || hFile == INVALID_HANDLE_VALUE)
	{
		CloseHandle(hFile);
		hFile = NULL;
	}
	else
	{
		if (bResume)
		{
			// GetfileSize
			LARGE_INTEGER FileStartPosition = {0};
			::GetFileSizeEx(hFile, &FileStartPosition);
			t_FileOffset = FileStartPosition.QuadPart;
		}

		if (m_pQTSHTTPFileStation->DownloadFileByPath(strDownloadFileFullPath, strToFilePath, t_FileOffset, pbCancel, lpData, pfHttpFSCallback) == QTS_HTTP_API_SUCCESS)
			result = TRUE;
	}

	if(hFile)
	{
		CloseHandle(hFile);
		hFile = NULL;
	}

	return result;
}

BOOL QTSHAPIWrapper::CopyAndRenameFile(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, LPCTSTR strRenameFilename, BOOL* pbCancel, LPVOID lpData, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback)
{
	BOOL result = FALSE;

	std::wstring ToPath(strToPath);
	std::wstring fromPath(strFromPath);
	std::wstring::size_type found = fromPath.find_last_of(L"/\\");
	std::wstring strfromFileName = fromPath.substr(found+1);
	std::wstring RenamePath = ToPath.append(L"/");
	RenamePath.append(strfromFileName);

	if (m_pQTSHTTPFileStation->CopyFileByPath(strFromPath, strToPath, nMode, pbCancel, lpData, pfHttpFSCallback) == QTS_HTTP_API_SUCCESS)
	{
		if (m_pQTSHTTPFileStation->RenameFile(RenamePath.c_str(), strRenameFilename, pbCancel) == QTS_HTTP_API_SUCCESS)
			result = TRUE;
	}

	return result;
}

BOOL QTSHAPIWrapper::MoveAndRenameFile(LPCTSTR strFromPath, LPCTSTR strToPath, int nMode, LPCTSTR strRenameFilename, BOOL* pbCancel, LPVOID lpData, PFQTS_HTTP_FILE_STATION_SYNC_PROGRESSRATE_CALLBACK pfHttpFSCallback)
{
	BOOL result = FALSE;

	std::wstring ToPath(strToPath);
	std::wstring fromPath(strFromPath);
	std::wstring::size_type found = fromPath.find_last_of(L"/\\");
	std::wstring strfromFileName = fromPath.substr(found+1);
	std::wstring RenamePath = ToPath.append(L"/");
	RenamePath.append(strfromFileName);

	if (m_pQTSHTTPFileStation->MoveFileByPath(strFromPath, strToPath, nMode, pbCancel, lpData, pfHttpFSCallback) == QTS_HTTP_API_SUCCESS)
	{
		if (m_pQTSHTTPFileStation->RenameFile(RenamePath.c_str(), strRenameFilename, pbCancel) == QTS_HTTP_API_SUCCESS)
			result = TRUE;
	}

	return result;
}

