
// QTSHttpDemoDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include "afxcmn.h"
#include "afxshelllistctrl.h"
#include "WaitDlg.h"

#include "QTSHAPIWrapper.h"

#define UPDATE_PROGRESS 501
#define ACTION_UPLOAD	0
#define ACTION_DOWNLOAD 1

#define QSYNC_PORT_MAPPING_START_INDEX             30000
#define QSYNC_PORT_MAPPING_END_INDEX               50000

#define WEB_SERVER_PORT                            80
#define SECURE_WEB_SERVER_PORT                     8081
#define SYSTEM_PORT                                8080
#define SECURE_SYSTEM_PORT                         443

#define QNAP_API_COMPLETE	6001
#define QNAP_SYS_API_COMPLETE	6002
#define QNAP_CLOUD_API_COMPLETE	6003

typedef struct _AUTH_DATA_
{
    char szUsername[MAX_PATH];
    char szPassword[MAX_PATH];
} AUTH_DATA, *PAUTH_DATA;

typedef struct _COPYFILE_ACTION
{
	int nAction;
	TCHAR LocalFile[MAX_PATH];
	TCHAR RemoteFile[MAX_PATH];
}COPYFILE_ACTION;

typedef enum {
	LOGIN,
	LOGIN_START,
	LOGOUT,
	LOGOUT_START,
	COPYFILE_START,
	COPYFILE,
	COPYFILE_FAIL,
	COPYFILE_DONE
}LOGCATEGORY;

typedef struct _FILE_INFO{
	TCHAR strFileName[MAX_PATH];
	BOOL  bFolder;
}FILE_INFO;

// CQTSHttpDemoDlg dialog
class CQTSHttpDemoDlg : public CDialogEx
{
// Construction
public:
	CQTSHttpDemoDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_QTSHTTPDEMO_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;
    static HANDLE m_hMutex;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()

	void Login();
	void Logout();
    QTSHAPIWrapper*		m_QTSHAPIWrapper;

public:
	COPYFILE_ACTION m_CopyAction;

	CEdit m_editUsername;
	CEdit m_editPassword;
	CStatic m_txtProgress;
    CProgressCtrl m_cCopyProgressbar;
    CIPAddressCtrl m_IPAddress;
    DWORD m_Port;
    BOOL m_Secure;

	CString m_strMyQNAPNAS;
	CString m_strUsername;
	CString m_strPassword;

	CButton m_btnLogin;
	CButton m_btnUpload;
	CButton m_btnDownload;

	CWaitDlg*	m_pWaiting;

	BOOL		m_bIsLogin;
    BOOL		m_bMapping;
	LPVOID		m_lpWFMAuth;
	CString		m_strLogFilename;
	CFile		m_fLogFile;
	CWinThread* m_pThread;

	DWORD	m_dwCopyStart;
	BOOL	m_bCancel;
	BOOL	m_bCanAddLog;

	CTreeCtrl m_LocalTreeCtrl;
	CListCtrl m_LocalListCtrl;
	CTreeCtrl m_RemoteTreeCtrl;
	CListCtrl m_RemoteListCtrl;
	CImageList m_ImageList;

	int m_nFileCount;
	LONGLONG m_llFileSize;

	void OnStartCopyFile(int nAction, CString LocalFile, CString RemoteFile);

	void UploadFile(BOOL* pbCancel = NULL);
	void DownloadFile(BOOL* pbCancel = NULL);
	void CopyFileCompleted();
    void SetProgress(int Progress);
	void AddLog(LOGCATEGORY category, CString LogContent1, CString LogContent2 = _T(""), CString LogContent3 = _T(""));
	
	static UINT LoginProc(IN LPVOID lpContext);
    static UINT LogoutProc(IN LPVOID lpContext);
	
    LARGE_INTEGER m_Progress;
	static UINT CopyFileProc(IN LPVOID lpContext);
    static void CALLBACK copy_file_progress_callback(LPVOID owner, LONGLONG llTotalBytesTransferred);
	
	void ExpandLocalTreeItem(HTREEITEM hItem);
	void ExpandRemoteTreeItem(HTREEITEM hItem);
	void LoadLocalFileList(CString strPath);
	void LoadRemoteFileList(CString strPath);
	
	void DestroyTree(CTreeCtrl& tree, HTREEITEM item);
	void BuildRemoteSubTree(HTREEITEM hItem, LPCTSTR Node);
	void BuildRemoteShareFolder();

    static void Lock();
    static void Unlock();

	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg void OnBnClickedBtnDownload();
	
	afx_msg void OnDestroy();
	afx_msg void OnBnClickedLogin();	
	afx_msg void OnBnClickedBtnUpload();

	afx_msg void OnSelchangedLocalTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnItemexpandingLocalTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnDblclkLocalTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnDblclkLocalList(NMHDR *pNMHDR, LRESULT *pResult);
	
	afx_msg void OnSelchangedRemoteTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnItemexpandingRemoteTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnDblclkRemoteTree(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnDblclkRemoteList(NMHDR *pNMHDR, LRESULT *pResult);
};
