﻿#if !defined(QTS_UDP_H)
#define QTS_UDP_H

#ifdef QTSUDP_EXPORTS
#define QTSUDPDLL_API __declspec(dllexport)
#define EXPIMP_TEMPLATE
#else
#define QTSUDPDLL_API __declspec(dllimport)
#define EXPIMP_TEMPLATE extern
#endif

#define	MAX_PROTOCOLS_STRING_BUF_SIZE		100

/*!
 * \brief
 * Remote QNAP Turbo NAS info
 * 
 */
typedef struct _tagRemoteNAS_ {
	/** NAS configure or not */
	BOOL	bConfigured;									
	/** NAS admin web port */
	int		nAdminWebPort;									
	/** NAS web server port */
	int		nWebServerPort;									
	/** NAS SSL web server port */
	int		nSecureWebServerPort;	
	/** NAS SSL admin web port */
	int		nSecureAdminWebPort;	
	/** NAS active MAC */
	char	ActiveMACValue[8];
	/** NAS group */
	TCHAR	Group[16];
	/** NAS MAC address */
	TCHAR	MACAddress[MAX_PROTOCOLS_STRING_BUF_SIZE +1];
	/** NAS IP address */
	TCHAR	IPAddress[16];		
	/** NAS external IP address */
	TCHAR	ExternalAddress[16];
	/** NAS DDNS */
	TCHAR	DDNS[MAX_PROTOCOLS_STRING_BUF_SIZE + 1];	
	/** NAS my cloud nas name */
	TCHAR	MyCloudNas[MAX_PROTOCOLS_STRING_BUF_SIZE + 1];  
	/**NAS display model name */
	TCHAR	DisplayModelName[MAX_PROTOCOLS_STRING_BUF_SIZE + 1];  
	/** NAS server name */
	TCHAR	ServerName[MAX_PROTOCOLS_STRING_BUF_SIZE + 1];  
	/** NAS FW version */
	TCHAR	Version[MAX_PROTOCOLS_STRING_BUF_SIZE + 1];		
} RemoteNAS, *PRemoteNAS;

/*!
 * \brief
 * Callback function for search NAS.
 * 
 * \param
 * [OUT] pInfo : Remote NAS info
 * \param
 * [OUT] lpData : callback function owner
 * 
 * \returns
 * n/a
 */
typedef void (__stdcall *PFQTS_UDP_SEARCH_RESPONSE_CALLBACK) (PRemoteNAS pInfo, LPVOID lpData);

/*!
 * \brief
 * QTS UDP service class.
 * 
 * 1. Support UDP broadcast.
 * 2. Developers can start/stop search QNAP Turbo NAS.
 */
class QTSUDPDLL_API CQtsUdpService
{
public:
	/** Constructor */
	CQtsUdpService();
	/** Deconstructor */
	~CQtsUdpService();

	/*!
	* \brief
	* Start search QNAP turbo NAS.
	* 
	* \param
	* [IN] lpData : callback function owner
	* \param
	* [IN] pfUDPResponseCallback : callback function for search NAS
	* 
	* \returns
	* TRUE : success
	* \returns
	* FALSE : fail
	*/
	BOOL StartSearchNAS(LPVOID lpData, PFQTS_UDP_SEARCH_RESPONSE_CALLBACK pfUDPResponseCallback);
	/*!
	* \brief
	* Stop search QNAP turbo NAS.
	* 
	* \returns
	* TRUE : success
	* \returns
	* FALSE : fail
	*/
	BOOL StopSearchNAS(void);
};

#endif //QTS_UDP_H